@@ -1,15 +1,14 @@
 /*
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation version 2
- * of the License.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation version 2.1 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  * Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
@@ -30,6 +29,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <time.h>
 #include <errno.h>
 
 #include <sys/ioctl.h>
@@ -39,30 +39,59 @@
 #include <stdlib.h> /* free */
 
 #include <libdvbv5/dvb-demux.h>
+#include <libdvbv5/dvb-dev.h>
+
+#define MAX_TIME		10	/* 1.0 seconds */
+
+#define xioctl(fh, request, arg...) ({					\
+	int __rc;							\
+	struct timespec __start, __end;					\
+									\
+	clock_gettime(CLOCK_MONOTONIC, &__start);			\
+	do {								\
+		__rc = ioctl(fh, request, ##arg);			\
+		if (__rc != -1)						\
+			break;						\
+		if ((errno != EINTR) && (errno != EAGAIN))		\
+			break;						\
+		clock_gettime(CLOCK_MONOTONIC, &__end);			\
+		if (__end.tv_sec * 10 + __end.tv_nsec / 100000000 >	\
+		    __start.tv_sec * 10 + __start.tv_nsec / 100000000 +	\
+		    MAX_TIME)						\
+			break;						\
+	} while (1);							\
+									\
+	__rc;								\
+})
 
 int dvb_dmx_open(int adapter, int demux)
 {
-	char* demux_name = NULL;
 	int fd_demux;
-	int r;
+	struct dvb_device *dvb;
+	struct dvb_dev_list *dvb_dev;
 
-	r = asprintf(&demux_name, "/dev/dvb/adapter%i/demux%i", adapter, demux );
-	if (r < 0)
+	dvb = dvb_dev_alloc();
+	dvb_dev_find(dvb, 0);
+	dvb_dev = dvb_dev_seek_by_sysname(dvb, adapter, demux, DVB_DEVICE_DEMUX);
+	if (!dvb_dev) {
+		dvb_dev_free(dvb);
 		return -1;
-	fd_demux = open( demux_name, O_RDWR | O_NONBLOCK );
-	free(demux_name);
+	}
+
+	fd_demux = open(dvb_dev->path, O_RDWR | O_NONBLOCK);
+	dvb_dev_free(dvb);
 	return fd_demux;
 }
 
 void dvb_dmx_close(int dmx_fd)
 {
-	(void) ioctl(dmx_fd, DMX_STOP);
-	close( dmx_fd);
+	(void)xioctl(dmx_fd, DMX_STOP);
+	close(dmx_fd);
 }
 
 void dvb_dmx_stop(int dmx_fd)
 {
-	(void) ioctl(dmx_fd, DMX_STOP);
+	(void)xioctl(dmx_fd, DMX_STOP);
 }
 
 int dvb_set_pesfilter(int dmxfd, int pid, dmx_pes_type_t type,
@@ -71,7 +100,7 @@ int dvb_set_pesfilter(int dmxfd, int pid, dmx_pes_type_t type,
 	struct dmx_pes_filter_params pesfilter;
 
 	if (buffersize) {
-		if (ioctl(dmxfd, DMX_SET_BUFFER_SIZE, buffersize) == -1)
+		if (xioctl(dmxfd, DMX_SET_BUFFER_SIZE, buffersize) == -1)
 			perror("DMX_SET_BUFFER_SIZE failed");
 	}
 
@@ -83,7 +112,7 @@ int dvb_set_pesfilter(int dmxfd, int pid, dmx_pes_type_t type,
 	pesfilter.pes_type = type;
 	pesfilter.flags = DMX_IMMEDIATE_START;
 
-	if (ioctl(dmxfd, DMX_SET_PES_FILTER, &pesfilter) == -1) {
+	if (xioctl(dmxfd, DMX_SET_PES_FILTER, &pesfilter) == -1) {
 		fprintf(stderr, "DMX_SET_PES_FILTER failed "
 		"(PID = 0x%04x): %d %m\n", pid, errno);
 		return -1;
@@ -116,7 +145,7 @@ int dvb_set_section_filter(int dmxfd, int pid, unsigned filtsize,
 
 	sctfilter.flags = flags;
 
-	if (ioctl(dmxfd, DMX_SET_FILTER, &sctfilter) == -1) {
+	if (xioctl(dmxfd, DMX_SET_FILTER, &sctfilter) == -1) {
 		fprintf(stderr, "DMX_SET_FILTER failed (PID = 0x%04x): %d %m\n",
 			pid, errno);
 		return -1;
@@ -142,7 +171,7 @@ int dvb_get_pmt_pid(int patfd, int sid)
 	f.timeout = 0;
 	f.flags = DMX_IMMEDIATE_START | DMX_CHECK_CRC;
 
-	if (ioctl(patfd, DMX_SET_FILTER, &f) == -1) {
+	if (xioctl(patfd, DMX_SET_FILTER, &f) == -1) {
 		perror("ioctl DMX_SET_FILTER failed");
 		return -1;
 	}
