@@ -98,6 +98,8 @@ static const struct v4lconvert_pixfmt supported_src_pixfmts[] = {
 	{ V4L2_PIX_FMT_YUYV,		16,	 5,	 4,	0 },
 	{ V4L2_PIX_FMT_YVYU,		16,	 5,	 4,	0 },
 	{ V4L2_PIX_FMT_UYVY,		16,	 5,	 4,	0 },
+	{ V4L2_PIX_FMT_NV16,		16,	 5,	 4,	1 },
+	{ V4L2_PIX_FMT_NV61,		16,	 5,	 4,	1 },
 	/* yuv 4:2:0 formats */
 	{ V4L2_PIX_FMT_SPCA501,		12,      6,	 3,	1 },
 	{ V4L2_PIX_FMT_SPCA505,		12,	 6,	 3,	1 },
@@ -140,6 +142,9 @@ static const struct v4lconvert_pixfmt supported_src_pixfmts[] = {
 	{ V4L2_PIX_FMT_Y10BPACK,	10,	20,	20,	0 },
 	{ V4L2_PIX_FMT_Y16,		16,	20,	20,	0 },
 	{ V4L2_PIX_FMT_Y16_BE,		16,	20,	20,	0 },
+	/* hsv formats */
+	{ V4L2_PIX_FMT_HSV32,		32,	 5,	 4,	0 },
+	{ V4L2_PIX_FMT_HSV24,		24,	 5,	 4,	0 },
 };
 
 static const struct v4lconvert_pixfmt supported_dst_pixfmts[] = {
@@ -1229,6 +1234,20 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 		}
 		break;
 
+	case V4L2_PIX_FMT_NV16: {
+		unsigned char *tmpbuf;
+
+		tmpbuf = v4lconvert_alloc_buffer(width * height * 2,
+				&data->convert_pixfmt_buf, &data->convert_pixfmt_buf_size);
+		if (!tmpbuf)
+			return v4lconvert_oom_error(data);
+
+		v4lconvert_nv16_to_yuyv(src, tmpbuf, width, height);
+		src_pix_fmt = V4L2_PIX_FMT_YUYV;
+		src = tmpbuf;
+		bytesperline = bytesperline * 2;
+		/* fall through */
+	}
 	case V4L2_PIX_FMT_YUYV:
 		if (src_size < (width * height * 2)) {
 			V4LCONVERT_ERR("short yuyv data frame\n");
@@ -1251,6 +1270,21 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 		}
 		break;
 
+	case V4L2_PIX_FMT_NV61: {
+		unsigned char *tmpbuf;
+
+		tmpbuf = v4lconvert_alloc_buffer(width * height * 2,
+				&data->convert_pixfmt_buf, &data->convert_pixfmt_buf_size);
+		if (!tmpbuf)
+			return v4lconvert_oom_error(data);
+
+		/* Note NV61 is NV16 with U and V swapped so this becomes yvyu. */
+		v4lconvert_nv16_to_yuyv(src, tmpbuf, width, height);
+		src_pix_fmt = V4L2_PIX_FMT_YVYU;
+		src = tmpbuf;
+		bytesperline = bytesperline * 2;
+		/* fall through */
+	}
 	case V4L2_PIX_FMT_YVYU:
 		if (src_size < (width * height * 2)) {
 			V4LCONVERT_ERR("short yvyu data frame\n");
@@ -1296,6 +1330,65 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 			break;
 		}
 		break;
+	case V4L2_PIX_FMT_HSV24:
+		if (src_size < (width * height * 3)) {
+			V4LCONVERT_ERR("short hsv24 data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
+		switch (dest_pix_fmt) {
+		case V4L2_PIX_FMT_RGB24:
+			v4lconvert_hsv_to_rgb24(src, dest, width, height, 0,
+						24, fmt->fmt.pix.hsv_enc);
+			break;
+		case V4L2_PIX_FMT_BGR24:
+			v4lconvert_hsv_to_rgb24(src, dest, width, height, 1,
+						24, fmt->fmt.pix.hsv_enc);
+			break;
+		case V4L2_PIX_FMT_YUV420:
+			v4lconvert_hsv_to_rgb24(src, dest, width, height, 0,
+						24, fmt->fmt.pix.hsv_enc);
+			v4lconvert_rgb24_to_yuv420(src, dest, fmt, 0, 0, 3);
+			break;
+		case V4L2_PIX_FMT_YVU420:
+			v4lconvert_hsv_to_rgb24(src, dest, width, height, 0,
+						24, fmt->fmt.pix.hsv_enc);
+			v4lconvert_rgb24_to_yuv420(src, dest, fmt, 0, 1, 3);
+			break;
+		}
+
+		break;
+
+	case V4L2_PIX_FMT_HSV32:
+		if (src_size < (width * height * 4)) {
+			V4LCONVERT_ERR("short hsv32 data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
+		switch (dest_pix_fmt) {
+		case V4L2_PIX_FMT_RGB24:
+			v4lconvert_hsv_to_rgb24(src, dest, width, height, 0,
+						32, fmt->fmt.pix.hsv_enc);
+			break;
+		case V4L2_PIX_FMT_BGR24:
+			v4lconvert_hsv_to_rgb24(src, dest, width, height, 1,
+						32, fmt->fmt.pix.hsv_enc);
+			break;
+		case V4L2_PIX_FMT_YUV420:
+			v4lconvert_hsv_to_rgb24(src, dest, width, height, 0,
+						32, fmt->fmt.pix.hsv_enc);
+			v4lconvert_rgb24_to_yuv420(src, dest, fmt, 0, 0, 3);
+			break;
+		case V4L2_PIX_FMT_YVU420:
+			v4lconvert_hsv_to_rgb24(src, dest, width, height, 0,
+						32, fmt->fmt.pix.hsv_enc);
+			v4lconvert_rgb24_to_yuv420(src, dest, fmt, 0, 1, 3);
+			break;
+		}
+
+		break;
+
+
 
 	default:
 		V4LCONVERT_ERR("Unknown src format in conversion\n");
