@@ -151,6 +151,7 @@ enum Option {
 	OptSetEdid,
 	OptClearEdid,
 	OptGetEdid,
+	OptInfoEdid,
 	OptFixEdidChecksums,
 	OptFreqSeek,
 	OptEncoderCmd,
@@ -169,12 +170,15 @@ enum Option {
 	OptStreamCount,
 	OptStreamSkip,
 	OptStreamLoop,
+	OptStreamSleep,
 	OptStreamPoll,
 	OptStreamTo,
+	OptStreamToHost,
 	OptStreamMmap,
 	OptStreamUser,
 	OptStreamDmaBuf,
 	OptStreamFrom,
+	OptStreamFromHost,
 	OptStreamOutPattern,
 	OptStreamOutSquare,
 	OptStreamOutBorder,
