@@ -224,6 +224,8 @@ static std::string cap2s(unsigned cap)
 		s += "\t\tRDS Output\n";
 	if (cap & V4L2_CAP_SDR_CAPTURE)
 		s += "\t\tSDR Capture\n";
+	if (cap & V4L2_CAP_TOUCH)
+		s += "\t\tTouch Device\n";
 	if (cap & V4L2_CAP_TUNER)
 		s += "\t\tTuner\n";
 	if (cap & V4L2_CAP_HW_FREQ_SEEK)
@@ -249,14 +251,26 @@ static std::string cap2s(unsigned cap)
 
 static void print_regs(int fd, struct v4l2_dbg_register *reg, unsigned long min, unsigned long max, int stride)
 {
-	unsigned long mask = stride > 1 ? 0x1f : 0x0f;
+	unsigned long mask;
 	unsigned long i;
 	int line = 0;
 
+	/* Query size of the first register */
+	reg->reg = min;
+	if (ioctl(fd, VIDIOC_DBG_G_REGISTER, reg) == 0) {
+		/* If size is set, then use this as the stride */
+		if (reg->size)
+			stride = reg->size;
+	}
+
+	mask = stride > 2 ? 0x1f : 0x0f;
+
 	for (i = min & ~mask; i <= max; i += stride) {
 		if ((i & mask) == 0 && line % 32 == 0) {
 			if (stride == 4)
 				printf("\n                00       04       08       0C       10       14       18       1C");
+			else if (stride == 2)
+				printf("\n            00   02   04   06   08   0A   0C   0E");
 			else
 				printf("\n          00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");
 		}
@@ -574,6 +588,12 @@ int main(int argc, char **argv)
 			usage();
 		set_reg.reg = parse_reg(curr_bd, reg_set_arg);
 		while (optind < argc) {
+			unsigned size = 0;
+
+			if (doioctl(fd, VIDIOC_DBG_G_REGISTER, &set_reg,
+				    "VIDIOC_DBG_G_REGISTER") >= 0)
+				size = set_reg.size;
+
 			set_reg.val = strtoull(argv[optind++], NULL, 0);
 			if (doioctl(fd, VIDIOC_DBG_S_REGISTER, &set_reg,
 						"VIDIOC_DBG_S_REGISTER") >= 0) {
@@ -591,7 +611,7 @@ int main(int argc, char **argv)
 				printf("Failed to set register 0x%08llx value 0x%llx: %s\n",
 					set_reg.reg, set_reg.val, strerror(errno));
 			}
-			set_reg.reg++;
+			set_reg.reg += size ? : (forcedstride ? : 1);
 		}
 	}
 
